/*
** $PROJECT: HyperText-DataType-System
**
** $VER: AddTOCEntry.rexx 40.3 (16.09.96)
**
** by
**
** Stefan Ruppert , Windthorststrasse 5 , 65439 Floersheim , GERMANY
**
** (C) Copyright 1996
** All Rights Reserved !
**
** $HISTORY:
**
** 16.09.96 : 040.003 : added some comments
** 31.08.96 : 040.002 : added NODENAME field
** 29.08.96 : 040.001 : initial
**
*/

Parse Arg title file nodename

Options Results

/* if no arguments are given from command line, try to get information
 * from MultiView. This uses the hypertext.datatype ARexx port.
 */
If title = '' | file = '' Then Do
   If ~Show(Ports,'MULTIVIEW.1.1') Then
   Do
      Say 'Multiview not started or displays a none HyperText object'
      Exit 10
   End

   ADDRESS 'MULTIVIEW.1.1'

   /* Obtain current object information */
   'GETATTR OBJECT=OBJECT FIELD=TITLE    VAR=TITLE'
   'GETATTR OBJECT=OBJECT FIELD=FILE     VAR=FILE'
   'GETATTR OBJECT=OBJECT FIELD=NODENAME VAR=NODENAME'
End

/* Get Global TOC file name */
tocfile = 'Help:' || GetEnv('Language') || '/HTDS_TOC.hguide'
tmpfile = 'T:ate' || Time(s)

file     = Strip(Trim(file))
title    = Strip(Trim(title))

filename = GetFileName(file)
letter   = Upper(Left(Strip(filename),1))

if Length(nodename) > 0 Then Do
   If Right(file,1) ~= ':' & Right(file,1) ~= '/' Then
      file = file || '/'
   file = file || nodename
End

/* add a entry in the global table of contents file, sorted by the initial
 * letter.
 *
 * Implemented as a state machine.
 */

state = ''
If Open('tmp',tmpfile,'w') Then Do
   If Open('toc',tocfile,'r') Then Do
      state = 'begin'
      Do While ~Eof('toc')
	 line = ReadLn('toc')

	 If opt = 'test' Then Do
	    Say 'State : ' || state
	    Say Line
	 End

	 Select
	    When state = 'begin' Then Do
	       /* search for the beginning of the link section */
	       If line = '@remark HTDS_TOC BEGIN' Then
		  state = 'table'
	       Call WriteLn('tmp',line)
	    End
	    When state = 'table' Then Do
	       /* now search for the appropriate initial letter */
	       If Left(line,11) = '@{m_letter ' Then Do
		  If SubStr(line,12,1) = letter Then
		     state = 'insert'
		  Else if SubStr(line,12,1) > letter Then Do
		     Call WriteLn('tmp','@{m_letter ' || letter || '}')
		     Call WriteLn('tmp','@{" ' || filename || ' " link "' || file || '"}@{tab}' || title)
		     Call WriteLn('tmp','')
		     state = 'end'
		  End

		  Call WriteLn('tmp',line)
	       End
	       Else If line = '@remark HTDS_TOC END' then Do
		  /* now entry found with the same letter, just append the current one */
		  Call WriteLn('tmp','@{m_letter ' || letter || '}')
		  Call WriteLn('tmp','@{" ' || filename || ' " link "' || file || '"}@{tab}' || title)
		  Call WriteLn('tmp','')
		  state = 'end'

		  Call WriteLn('tmp',line)
	       End
	       Else
		  Call WriteLn('tmp',line)
	    End
	    When state = 'insert' Then Do
	       /* there exist some entries with the same initial letter, now sort
		* it within these entries
		*/

	       Parse Var line '@{" ' linefilename ' " link "' linkname '"}@{tab}' rest
	       rest = Strip(Trim(rest))
	       linefilename = Strip(Trim(linefilename))
	       If line = '' | Upper(filename) <= Upper(linefilename) Then Do
		  If (linefilename ~= filename) | (rest ~= title) Then
		     Call WriteLn('tmp','@{" ' || filename || ' " link "' || file || '"}@{tab}' || title)
		  state = 'end'
	       End
	       Call WriteLn('tmp',line)
	    End
	    When state = 'end'   Then Do
	       Call WriteLn('tmp',line)
	    End
	 End
      End
      Call Close('toc')
   End
   Call Close('tmp')
End
If state = 'end' Then
   Address Command 'Copy ' || tmpfile || ' TO ' || tocfile ' QUIET'

Address Command 'Delete ' || tmpfile || ' QUIET'
Exit

/* GetEnv(var) function */
GetEnv:
   var = Arg(1)
   contents = ''
   if Open("file","Env:" || var,"R") then do
      contents = readln("file")
      dummy = close("file")
   end

return contents

GetFileName:
   file = Arg(1)
   pos  = Max(LastPos(':', file),LastPos('/', file)) + 1
   return SubStr(file,pos)

