#ifndef DATATYPES_DIRECTORYCLASS_H
#define DATATYPES_DIRECTORYCLASS_H
/*
** $PROJECT: HyperText-DataType-System
**
** $VER: directoryclass.h 40.2 (08.09.96)
**
** by
**
** Stefan Ruppert , Windthorststrasse 5 , 65439 Floersheim , GERMANY
**
** (C) Copyright 1996
** All Rights Reserved !
**
*/

/* ------------------------------- includes ------------------------------- */

#ifndef UTILITY_TAGITEM_H
#include <utility/tagitem.h>
#endif

#ifndef  DATATYPES_DATATYPESCLASS_H
#include <datatypes/datatypesclass.h>
#endif

#ifndef DATATYPES_HYPERTEXTCLASS_H
#include <datatypes/hypertextclass.h>
#endif

/* ------------------------------ class name ------------------------------ */

#define DIRECTORYDTCLASS         "directory.datatype"
#define DIRECTORYDTLIBRARY       "datatypes/directory.datatype"
#define DIRECTORYDTVERSION       40

/* ------------------------------- methods -------------------------------- */

#define DIRDTM_Base              0x900

#define DIRDTM_INSERTENTRY       0x901

/* ------------------------------- messages ------------------------------- */

/* DIRDTM_INSERTENTRY */
struct dirInsertEntry
{
   ULONG MethodID;
   struct GadgetInfo *dt_GInfo;
   struct DirDTEntry *dir_Entry;
};

/* ------------------------------ attributes ------------------------------ */

#define DIRDTA_Dummy             (TAG_USER + 40200)

#define DIRDTA_Format            (DIRDTA_Dummy + 1)
			/* (STRPTR) format string for formatting each entry
			 * Defaults to :
			 * "%s @{fg highlight}%l@{fg text} %a @{b}%d@{ub} %t @{jright}%z@{jleft}"
			 *
			 * Applicability is (ISG)
			 */

#define DIRDTA_Entries           (DIRDTA_Dummy + 2)
			/* (struct List *) list of (struct DirDTEntry *) entries
			 * Applicability is (G)
			 */

#define DIRDTA_SortBy            (DIRDTA_Dummy + 3)
			/* (ULONG) defines the sort criteria. See DIRSB_#?
			 * values
			 * Defaults to DIRSB_NAME.
			 *
			 * Applicability is (ISG)
			 */

#define DIRDTA_ShowType          (DIRDTA_Dummy + 4)
			/* (ULONG) defines, how to show the directory. See
			 * DIRST_#? values.
			 * Defaults to DIRST_DIRSFIRST.
			 *
			 * Applicability is (ISG).
			 */

#define DIRDTA_HideInfo          (DIRDTA_Dummy + 5)
			/* (BOOL) indicates, if icon files should be listed
			 * (FALSE) or should not (TRUE)
			 * Defaults to FALSE.
			 *
			 * Applicability is (ISG)
			 */

#define DIRDTA_HideDot           (DIRDTA_Dummy + 6)
			/* (BOOL) indicates, if ".#?" files should be listed
			 * (FALSE) or should not (TRUE). Unix compatibility.
			 * Defaults to FALSE.
			 *
			 * Applicability is (ISG)
			 */

#define DIRDTA_ParentEntry       (DIRDTA_Dummy + 7)
			/* (BOOL) indicates, if a parent "/" entry should be
			 * used (TRUE) or not (FALSE).
			 * Defaults to TRUE.
			 *
			 * Applicability is (ISG)
			 */

#define DIRDTA_UseDTLib          (DIRDTA_Dummy + 8)
			/* (BOOL) indecates, if the datatypes.library should
			 * be used to determine the type of the entry.
			 * If this is TRUE each entry is checked via
			 * ObtainDataTypesA().
			 * If its FALSE, only thd DirEntryType is checked
			 * and the DataType field is set to File,Dir,HardLink
			 * etc.
			 * Default is FALSE.
			 *
			 * Applicability is (IG).
			 */

/* ------------------------------- defines -------------------------------- */

/* DIRDTA_SortBy values */
#define DIRSB_NAME         1
#define DIRSB_LENGTH       2
#define DIRSB_DATE         3

/* DIRDTA_ShowType values */
#define DIRST_ICONS        1     /* not implemented */
#define DIRST_DIRSFIRST    2
#define DIRST_DIRSLAST     3
#define DIRST_MIXED        4

/* ------------------------------ structures ------------------------------ */

struct DirDTEntry
{
   struct MinNode de_Node;
   STRPTR de_FileName;
   LONG de_DirEntryType;
   LONG de_Protection;
   LONG de_Size;
   STRPTR de_Comment;
   struct DateStamp de_Date;
   UWORD de_OwnerID;
   UWORD de_OwnerGID;
   STRPTR de_DataType;
};

#endif /* !DATATYPES_DIRECTORYCLASS_H */

