/*
** $PROJECT: catalog.datatype DataType for locale files
**
** $VER: dispatch.c 40.1 (08.09.96)
**
** by
**
** Stefan Ruppert , Windthorststrasse 5 , 65439 Floersheim , GERMANY
**
** (C) Copyright 1996
** All Rights Reserved !
**
** $HISTORY:
**
** 08.09.96 : 040.001 : initial
*/

/*FS*//* ------------------------------- include -------------------------------- */

#include "classbase.h"
/*FE*/

/*FS*//* ------------------------------- autodoc -------------------------------- */

/****** catalog.datatype/catalog.datatype ************************************
*
*    NAME
*        catalog.datatype -- Datatype for locale catalog files.
*
*    FUNCTION
*        This datatype loads any locale catalog file and can display it
*        using DataType compatible programs like MultiView.
*
*    METHODS
*        No own methods. The following methods overwrite the ones from the
*        SuperClass :
*
*        OM_SET/OM_UPDATE -- these method checks, if the object have to
*            be rendered and performs it, if necessary.
*
*        GM_LAYOUT/DTM_PROCLAYOUT and DTM_ASYNCLAYOUT -- these methods
*            tries to layout the object data and for the first time, it
*            also loads the data.
*
*    SEE ALSO
*        hypertext.datatype, Source code
*
******************************************************************************
*
*/
/*FE*/

/*FS*//* ------------------------------- defines -------------------------------- */

#define BUFFER_SIZE        512

#define GAD(o)             ((struct Gadget *) (o))
#define GPL(msg)           ((struct gpLayout *) (msg))
#define SET(msg)           ((struct opSet *) (msg))

#define EOS                '\0'

#define LANG_ID            MAKE_ID('L','A','N','G')
#define STRS_ID            MAKE_ID('S','T','R','S')
/*FE*/

/*FS*//* ----------------------------- Preferences ------------------------------ */

/**** Prefs file ReadArgs() support
#define PREFS_TEMPLATE        "MYOPT/K,MYSWITCH/S,MYNUMBER/N"
struct Args
{
   STRPTR a_MyOpt;
   LONG a_MySwitch;
   LONG* a_MyNumber;
};
****/

#ifdef PREFS_TEMPLATE
static BOOL ReadPrefs(Class *cl,Object *obj,STRPTR file)
{
   struct ClassBase *cb = (struct ClassBase *) cl->cl_UserData;
   struct DirData *data = INST_DATA(cl,obj);
   struct RDArgs *rdargs;
   struct RDArgs *args;
   BPTR fh;

   if((fh = Open(file,MODE_OLDFILE)) != NULL)
   {
      UBYTE buf[256];

      DB(("prefs file \"%s\" opened\n",file));
      if((rdargs = (struct RDArgs *) AllocDosObject(DOS_RDARGS,NULL)) != NULL)
      {
	 while(FGets(fh,buf,sizeof(buf)) != NULL)
	 {
	    struct Args para;
	    /* skip comment and empty lines, to speed up parsing */
	    if(buf[0] == ';' || buf[0] == '\n')
	       continue;

	    rdargs->RDA_Source.CS_Buffer = buf;
	    rdargs->RDA_Source.CS_Length = strlen(buf);
	    rdargs->RDA_Source.CS_CurChr = 0;

	    memset(&para,0,sizeof(struct Args));

	    if((args = ReadArgs(PREFS_TEMPLATE,(LONG *) &para,rdargs)) != NULL)
	    {
	       DB(("args at %lx\n",args));

	       if(para.a_MyOpt != NULL)
	       {
	       }

	       if(para.a_MySwitch)
	       {
	       }

	       if(para.a_MyNumber != NULL)
	       {
	       }

	       FreeArgs(args);
	    }
	 }
	 FreeDosObject(DOS_RDARGS , rdargs);
      }
      Close(fh);
   }

   return((BOOL) (fh == NULL));
}
#endif
/*FE*/

/*FS*//* ---------------------------- notify object ----------------------------- */

ULONG NotifyAttrs(Object *o,void *ginfo,ULONG flags,ULONG tag1,...)
{
    return(DoMethod(o, OM_NOTIFY, &tag1, ginfo, flags));
}
/*FE*/

/*FS*//* --------------------------- class dispatcher --------------------------- */

ClassCall ULONG dispatcher(REGA0 Class *cl,REGA2 Object *obj,REGA1 Msg msg)
{
   struct ClassBase *cb = (struct ClassBase *) cl->cl_UserData;
   ULONG retval = 0;

   switch(msg->MethodID)
   {
   case OM_NEW:
      {
	 Object *newobj;

	 if((newobj = (Object *) DoSuperMethodA(cl,obj,msg)) != NULL)
	 {
	    DB(("After OM_NEW DoSuperMethodA()\n"));

	    /* initialize the instance data here */

#ifdef PREFS_TEMPLATE
	    if(PROC(FindTask(NULL)->pr_HomeDir == NULL || ReadPrefs(cl,newobj,"PROGDIR:Prefs/DataTypes/xxx.prefs"))
	       ReadPrefs(cl,newobj,"ENV:DataTypes/xxx.prefs");
#endif

	    retval = (ULONG) newobj;
	 }
      }
      break;
   case OM_DISPOSE:
      /* don't need to free any resource, if you use only the SuperClass
       * memory pool (HTDTA_Pool).
       */
      retval = DoSuperMethodA(cl,obj,msg);
      break;
   case OM_UPDATE:
   case OM_SET:
      /* Pass the attributes to the hypertext class and force a refresh
       * if we need it */
      if((retval = DoSuperMethodA (cl, obj, msg)) && (OCLASS (obj) == cl))
      {
	 struct RastPort *rp;

	 if((rp = ObtainGIRPort(SET(msg)->ops_GInfo)) != NULL)
	 {
	    /* send only a update message GREDRAW_UPDATE */
	    DoMethod(obj,GM_RENDER,SET(msg)->ops_GInfo,rp,GREDRAW_UPDATE);

	    ReleaseGIRPort (rp);
	 }

	 retval = 0;
      }
      break;
   case GM_LAYOUT:
      /* Tell everyone that we are busy doing things */
      NotifyAttrs (obj, GPL(msg)->gpl_GInfo, NULL,
			GA_ID,       GAD(obj)->GadgetID,
			DTA_Busy,    TRUE,
			TAG_DONE);

      /* NOTE: this is different from current known datatypes implementations
       * but its necessary! The hypertext.datatype depends on it.
       * So only do a AsyncLayout, when the SuperClass returns a zero.
       */
      if((retval = DoSuperMethodA (cl, obj, msg)) == 0)
      {
	 /* We need to do this one asynchronously */
	 retval += DoAsyncLayout (obj, (struct gpLayout *) msg);
      }
      break;
   case DTM_PROCLAYOUT:
      /* Tell everyone that we are busy doing things */
      NotifyAttrs (obj, GPL(msg)->gpl_GInfo, NULL,
			GA_ID,       GAD(obj)->GadgetID,
			DTA_Busy,    TRUE,
			TAG_DONE);

      /* Let the super-class partake and then fall through to our layout method */
      DoSuperMethodA (cl, obj, msg);
   case DTM_ASYNCLAYOUT:
      /* Layout the text */
      retval = layout(cl, obj, GPL(msg));
      break;
   default:
      retval = DoSuperMethodA(cl,obj,msg);
   }

   return retval;
}
/*FE*/

/*FS*//* -------------------------------- layout -------------------------------- */

ULONG layout(Class *cl, Object *obj, struct gpLayout *msg)
{
   struct ClassBase *cb = (struct ClassBase *) cl->cl_UserData;
   struct DataType *dt;
   struct IFFHandle *handle;
   APTR pool;
   ULONG maxwidth = 0;
   ULONG total    = 0;
   ULONG th = 0,tv = 0;

   /* attributes obtained from super-class */
   struct IBox *domain;
   struct TextFont *font;
   STRPTR title;

   DB(("catalog layout , obj : %lx!\n",obj));

   /* Get all the attributes that we are going to need for a successful layout */
   if(GetDTAttrs(obj,
		 DTA_TextFont,      (ULONG) &font,
		 DTA_ObjName,       (ULONG) &title,
		 DTA_Domain,        (ULONG) &domain,
		 DTA_TopHoriz,      (ULONG) &th,
		 DTA_TopVert,       (ULONG) &tv,
		 DTA_Handle,        (ULONG) &handle,
		 DTA_DataType,      (ULONG) &dt,
		 HTDTA_Pool,        (ULONG) &pool,
		 TAG_DONE) == 8 && pool != NULL)
   {
      ULONG hunit,vunit;
      ULONG hvis,vvis;

      BOOL abort = FALSE;
      struct HTSegInfo *seginfo;

      hunit = font->tf_XSize;
      vunit = font->tf_YSize;
      hvis  = domain->Width  / hunit;
      vvis  = domain->Height / vunit;

      DB(("before OBTAIN object\n"));
      /* lock datatype object */
      DoMethod(obj,HTDTM_OBTAIN);

      if((dt->dtn_Header->dth_Flags & DTF_TYPE_MASK) == DTF_IFF)
      {
	 const ULONG stopchunks[] =
	 {
	     MAKE_ID('C','T','L','G'),LANG_ID,
	     MAKE_ID('C','T','L','G'),STRS_ID
	 };

	 DB(("before AllocSegInfo()\n"));
	 /* We only need to perform layout if this is the initial layout call. We
	  * need also a SegInfo structure !
	  */
	 if(msg->gpl_Initial && (seginfo = AllocSegInfo(HTDTSIA_Pool,pool,
							HTDTSIA_Object,obj,
							HTDTSIA_GadgetInfo,msg->gpl_GInfo,
							TAG_DONE)) != NULL)
	 {
	    struct HTConvInfo *convinfo;
	    struct TagItem tags[5];
	    STRPTR fmtbuf;
	    STRPTR buf;

	    DB(("before AllocPooled()\n"));
	    if((buf = AllocPooled(pool,2*BUFFER_SIZE)) != NULL)
	    {
	       fmtbuf = buf + BUFFER_SIZE;

	       /* AllocConvInfoA() set up defaults */
	       tags[0].ti_Tag  = HTDTSA_LayoutFlags;
	       tags[1].ti_Tag  = HTDTSA_FgPen;
	       tags[2].ti_Tag  = HTDTSA_BgPen;
	       tags[3].ti_Tag  = HTDTSA_Style;
	       tags[4].ti_Tag  = TAG_DONE;

	       convinfo = AllocConvInfoA(seginfo,tags,NULL);

	       DB(("handle : %lx\n",handle));
	       DB(("before StopChunks()\n"));

	       if(StopChunks(handle,(LONG *) stopchunks,2) == 0)
	       {
		  STRPTR parsestring;
		  LONG rc;

		  struct StringControl {
		     LONG s_TextID;
		     LONG s_Len;
		  } strctl;

		  DB(("before ParseIFF()\n"));

		  while((rc = ParseIFF(handle,IFFPARSE_SCAN)) == 0)
		  {
		     struct ContextNode *top = CurrentChunk(handle);

		     DB(("found chuck : 0x%08lx\n",top->cn_ID));
		     switch(top->cn_ID)
		     {
		     case LANG_ID:
			parsestring = "Language : @{fg highlight}%s@{fg text}";

			AllocSegment(seginfo,HTST_NORMAL,
				     HTDTSA_Flags,HTSF_LF,
				     TAG_DONE);

			DB(("Language ID found!\n"));
			if(ReadChunkBytes(handle,buf,top->cn_Size) == top->cn_Size)
			{
			   STRPTR fmt = fmtbuf;
			   STRPTR seg,endseg;

			   sprintf(fmtbuf,parsestring,buf);

			   seg = fmt;
			   endseg = fmt;
			   while(*fmt != '\0')
			   {
			      if(*fmt == '@' && *(fmt + 1) == '{' && convinfo != NULL)
			      {
				 if(seg < endseg)
				    AllocSegment(seginfo,HTST_NORMAL,
						 HTDTSA_Text,seg,
						 HTDTSA_TextLen,endseg-seg,
						 HTDTSA_AllocText,TRUE,
						 TAG_MORE,tags);

				 fmt = ConvertAttributes(convinfo,fmt,strlen(fmt));
				 seg = fmt;
			      } else
				 fmt++;
			      endseg = fmt;
			   }

			   /* if(seg < endseg) disabled, because we need a linefeed */

			      AllocSegment(seginfo,HTST_NORMAL,
					   HTDTSA_Text,seg,
					   HTDTSA_TextLen,endseg-seg,
					   HTDTSA_Flags,HTSF_LF,
					   HTDTSA_AllocText,TRUE,
					   TAG_MORE,tags);
			}
			break;
		     case STRS_ID:
			AllocSegment(seginfo,HTST_NORMAL,
				     HTDTSA_Flags,HTSF_LF,
				     TAG_DONE);

			AllocSegment(seginfo,HTST_SEPARATOR,
				     TAG_DONE);

			AllocSegment(seginfo,HTST_NORMAL,
				     HTDTSA_Flags,HTSF_LF,
				     TAG_DONE);

			/* Note: id format must be first. See sprintf() below */
			parsestring = "TextID @{b}%08ld@{ub} : @{fg highlight}%s@{fg text}";

			DB(("found id chunk\n"));

			while(ReadChunkBytes(handle,&strctl,sizeof(struct StringControl)) == sizeof(struct StringControl))
			{
			   LONG len = (strctl.s_Len + 3) & ~(3);

			   if(ReadChunkBytes(handle,buf,len) == len)
			   {
			      STRPTR fmt = fmtbuf;
			      STRPTR seg,endseg;

			      sprintf(fmtbuf,parsestring,strctl.s_TextID,buf);

			      seg = fmt;
			      endseg = fmt;
			      while(*fmt != '\0')
			      {
				 if(*fmt == '@' && *(fmt + 1) == '{' && convinfo != NULL)
				 {
				    if(seg < endseg)
				       AllocSegment(seginfo,HTST_NORMAL,
						    HTDTSA_Text,seg,
						    HTDTSA_TextLen,endseg-seg,
						    HTDTSA_AllocText,TRUE,
						    TAG_MORE,tags);

				    fmt = ConvertAttributes(convinfo,fmt,strlen(fmt));
				    seg = fmt;
				 } else
				    fmt++;
				 endseg = fmt;
			      }

			      /* if(seg < endseg) disabled, because we need a linefeed */

				 AllocSegment(seginfo,HTST_NORMAL,
					      HTDTSA_Text,seg,
					      HTDTSA_TextLen,endseg-seg,
					      HTDTSA_Flags,HTSF_LF,
					      HTDTSA_AllocText,TRUE,
					      TAG_MORE,tags);
			   }
			}
			break;
		     }
		  }
	       }

	       if(convinfo != NULL)
		  FreeConvInfo(convinfo);

	       FreePooled(pool,buf,2*BUFFER_SIZE);
	    }
	    FreeSegInfo(seginfo);
	 }
      }

      /* these values must be set explicitly
       * possible error in datatypesclass ?
       */
      {
	 struct DTSpecialInfo *si = (struct DTSpecialInfo *) GAD(obj)->SpecialInfo;

	 si->si_VisVert  = vvis;
	 si->si_TotVert  = total;

	 si->si_VisHoriz = hvis;
	 si->si_TotHoriz = maxwidth;
	 si->si_VertUnit = vunit;
	 si->si_HorizUnit= hunit;
      }

      if(abort == FALSE)
      {
	 /* layout the linked list */
	 total = DoMethod(obj,HTDTM_LAYOUT,msg->gpl_GInfo,NULL,msg->gpl_Initial);

	 GetAttr(DTA_TotalHoriz,obj,&maxwidth);
      }

      /* release (hypertext) datatype object */
      DoMethod(obj,HTDTM_RELEASE);

      DB(("after release()\n"));

      /* Were we aborted? */
      if(abort == FALSE)
      {
	 /* Not aborted, so tell the world of our newest attributes */
	 NotifyAttrs(obj, msg->gpl_GInfo, 0,
			  GA_ID,                   GAD(obj)->GadgetID,

			  DTA_VisibleVert,         vvis,
			  DTA_TotalVert,           total,
			  DTA_NominalVert,         25 * vunit,
			  DTA_VertUnit,            vunit,

			  DTA_VisibleHoriz,        hvis,
			  DTA_TotalHoriz,          maxwidth,
			  DTA_NominalHoriz,        60 * hunit,
			  DTA_HorizUnit,           hunit,

			  DTA_TopHoriz,            th,
			  DTA_TopVert,             tv,

			  DTA_Title,               title,
			  DTA_Busy,                FALSE,
			  DTA_Sync,                TRUE,
			  TAG_DONE);
      }
   }

   return total;
}
/*FE*/

