#ifndef DEBUG_H
#define DEBUG_H

/*
** $PROJECT: C debugging macros
**
** $VER: debug.h 1.2 (14.08.96)
**
** by
**
** Stefan Ruppert , Windthorststrasse 5 , 65439 Floersheim , GERMANY
**
** (C) Copyright 1994,1995
** All Rights Reserved !
**
** $HISTORY:
**
** 14.08.96 : 001.002 : added OnlineDebugControl
** 02.09.95 : 001.001 : added assert like macro DA()
** 31.03.94 : 000.001 : initial
*/

#ifdef __cplusplus
extern "C" {
#endif

#ifdef DEBUG

#ifdef __SASC
#define DEBUGFUNC      __FUNC__
#elif __GNUC__
#define DEBUGFUNC      __FUNCTION__
#else
#define DEBUGFUNC
#endif

extern void KPrintF(const char *fmt,...);

#define D(x)                  x
#define bug                   KPrintF
#define fbug                  KPrintF
#define DEBUG_TAGLIST(x)      { struct TagItem *tstate = x; \
										  struct TagItem *tag; \
										  bug("TagList : \n"); \
										  while((tag = NextTagItem(&tstate))) \
											 bug("{0x%08lx,0x%08lx}\n",tag->ti_Tag,tag->ti_Data); \
										}
#define DEBUG_EXECLIST(x)     { struct Node *node; bug("ExecList : \n"); \
										  for(node = (x)->lh_Head; node->ln_Succ != NULL; node = node->ln_Succ) \
											  bug("0x%08lx : %s\n",node,node->ln_Name); \
										}


#ifdef DEBUG_ODC
#include "odc.h"

#define DC(x)     x
#define DB(x)     { ODC_OUTPUT(x); }
#define ENTERING  { ODC_FOUTPUT("entering"); }
#define LEAVING   { ODC_FOUTPUT("leaving");  }
#define DTL(x)    { ODC_BUG(DEBUG_TAGLIST(x)); }
#define DDL(x)    { ODC_BUG(DEBUG_EXECLIST(x)); }
#define DA(x,e)   { ODC_BUG(if(!(e)) { bug("Fault: "); bug x; }); }
#else

#define DBLINE    bug(PROJECTNAME " " __FILE__ "(%4ld):" DEBUGFUNC "() :",__LINE__)

#define DC(x)
#define DB(x)     { DBLINE; bug x; }
#define ENTERING  fbug("entering " DEBUGFUNC "()\n")
#define LEAVING   fbug("leaving " DEBUGFUNC "()\n")
#define DTL(x)    { bug(__FILE__ "(%4ld):" DEBUGFUNC "() ",__LINE__); \
						  DEBUG_TAGLIST(x); \
						}
#define DDL(x)    { bug(__FILE__ "(%4ld):" DEBUGFUNC "() ",__LINE__); \
						  DEBUG_EXECLIST(x); \
						}
#define DA(x,e)   { if(!(e)) { bug("Fault: "); bug x; } }
#endif

#ifndef PROJECTNAME
#define PROJECTNAME
#endif

/* assert like macro */

#define DELAY(x)     {if(FindTask(NULL)->tc_Node.ln_Type == NT_PROCESS) Delay(x);}

#else

#define bug
#define DC(x)
#define D(x)
#define DB(x)
#define DA(x,expr)
#define DELAY(x)
#define DTL(x)
#define DDL(x)
#define ENTERING
#define LEAVING

#endif

#ifdef __cplusplus
};
#endif

#endif   /* DEBUG_H */

