#ifndef ODC_H
#define ODC_H

/*
** $PROJECT: OnlineDebugControl (ODC)
**
** $VER: odc.h 1.2 (17.08.96)
**
** by
**
** Stefan Ruppert , Windthorststrasse 5 , 65439 Floersheim , GERMANY
**
** (C) Copyright 1994,1995
** All Rights Reserved !
**
** $HISTORY:
**
** 17.08.96 : 001.002 : added level support
** 14.08.96 : 001.001 : initial
*/

#ifdef __cplusplus
extern "C" {
#endif

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif

/* --------------------- debug information structure ---------------------- */

struct DebugNode
{
	UWORD dn_Level;
	UWORD dn_CallLevel;
	ULONG dn_Flags;
};

#define DNF_OUTPUT_PROJECT (1<<0)
#define DNF_OUTPUT_MODUL   (1<<1)
#define DNF_OUTPUT_LINE    (1<<2)

/* ------------------------------ ODC macros ------------------------------ */

#ifndef PROJECTNAME
#define ODC_PROJECTNAME "default"
#else
#define ODC_PROJECTNAME PROJECTNAME
#endif

#define ODC_OUTPUTLVL(lvl,x)   if(ODC_OutputDebug(ODC_PROJECTNAME,__FILE__,__LINE__,lvl)) \
										 { \
											 KPrintF(DEBUGFUNC "() :"); \
											 KPrintF x; \
										 }

#define ODC_FOUTPUTLVL(lvl,x)  if(ODC_OutputDebug(ODC_PROJECTNAME,__FILE__,__LINE__,lvl)) \
											 KPrintF(x DEBUGFUNC "() :");


#define ODC_OUTPUT(x)          ODC_OUTPUTLVL(1,x)
#define ODC_FOUTPUT(x)         ODC_FOUTPUTLVL(1,x)

#define ODC_BUGLVL(lvl,x)      if(ODC_OutputDebug(ODC_PROJECTNAME,__FILE__,__LINE__,lvl)) { x }
#define ODC_BUG(x)             ODC_BUGLVL(1,x)

#define ODC_SetLevel(level)      {  struct DebugNode *debug; \
												if((debug = ODC_ObtainDebugNode(ODC_PROJECTNAME,__FILE__)) != NULL) \
												{ \
													debug->dn_Level = level; \
													ODC_ReleaseDebugNode(debug); \
												} \
											}

#define ODC_SetCallLevel(level)  {  struct DebugNode *debug; \
												if((debug = ODC_ObtainDebugNode(ODC_PROJECTNAME,__FILE__)) != NULL) \
												{ \
													debug->dn_CallLevel = level; \
													ODC_ReleaseDebugNode(debug); \
												} \
											}

/* ------------------------------ prototypes ------------------------------ */

BOOL ODC_FOutputDebug(const STRPTR projname,const STRPTR modulname,const ULONG line,const LONG level);
BOOL ODC_OutputDebug(const STRPTR projname,const STRPTR modulname,const ULONG line,const LONG level);
struct DebugNode *ODC_ObtainDebugNode(const STRPTR projname,const STRPTR modulname);
void ODC_ReleaseDebugNode(struct DebugNode *debug);

#ifdef __cplusplus
};
#endif

#endif   /* ODC_H */

