/****************************************************************
 *
 * Name : PlayDT.c
 *
 * Author : Tak Tang
 *
 * Date : 2.1.97
 *
 * Usage : PlayDT FILE/A/M
 *
 * Description : Plays sound samples using datatypes
 *
 *****************************************************************/


/******** Soft definitions */

#define TEMPLATE        "FILE/A/M"
#define OPT_FILE        0
#define OPT_COUNT       1

#define MSG_FAILED "PlayDT failed"


/******** Header files */

#include <workbench/startup.h>
#include <workbench/workbench.h>

#include <datatypes/datatypes.h>
#include <datatypes/datatypesclass.h>
#include <datatypes/soundclass.h>

#include <clib/alib_protos.h>
#include <clib/dos_protos.h>
#include <clib/exec_protos.h>
#include <clib/datatypes_protos.h>

#include <pragmas/dos_pragmas.h>
#include <pragmas/exec_pragmas.h>
#include <pragmas/datatypes_pragmas.h>

#include <string.h>


/******** Type definitions */

struct tLocalData {
  struct Library *DataTypesBase;
//  struct Library *IntuitionBase;
  BOOL FromWB;
};


/******** Storage */

UBYTE VersTag[]="\0$VER: PlayDT 1.1 (2.1.97)";

extern struct ExecBase *SysBase;
extern struct Library  *DOSBase;
struct Library *DataTypesBase=NULL;


/******** Aux functions */

void PlayDT(struct tLocalData *ld, char *file) {
  Object *o;
  struct dtTrigger dtt;

  if (o = NewDTObject ((APTR)file,
      DTA_SourceType,        DTST_FILE,
      DTA_GroupID,           GID_SOUND,
      SDTA_Volume,           64,
      SDTA_Cycles,           1,
      SDTA_SignalTask,       (ULONG) FindTask (NULL),
      SDTA_SignalBit,        (ULONG) SIGBREAKB_CTRL_F,
      TAG_DONE))
  {

    dtt.MethodID     = DTM_TRIGGER;
    dtt.dtt_GInfo    = NULL;
    dtt.dtt_Function = STM_PLAY;
    dtt.dtt_Data     = NULL;

    /* Play the sound */
    DoMethodA (o, &dtt);

    Wait (SIGBREAKB_CTRL_F);

    /* Get rid of the object */
    DisposeDTObject (o);
  } else { // NewDTObject() failed
    if (ld->FromWB) {
      // should output an error message via a requester
    } else {
      LONG errnum;
      BPTR op;
      UBYTE errbuff[80];

      op = Output();
      errnum = IoErr ();
      FPrintf(op,"%s: ", MSG_FAILED);
      if (errnum >= DTERROR_UNKNOWN_DATATYPE) {
	FPrintf(op,GetDTString (errnum), file);
      } else {
	Fault (errnum, NULL, errbuff, sizeof (errbuff));
	FPrintf(op,"%s",errbuff);
      }
      FPrintf(op,"\nerror #%ld\n", errnum);
    } // if FromWB
  } // if NewDTObject
} // PlayDT


/******** Main entry */

void main(int argc, char **argv) {
  struct tLocalData ld;

  memset ( &ld, 0, sizeof(ld));

  if (ld.DataTypesBase = OpenLibrary ("datatypes.library", 39L)) {
    DataTypesBase = ld.DataTypesBase;

    if (0==argc) { // run from workbench
      struct WBArg *wba;
      struct WBStartup *wbs;
      long i;

      ld.FromWB=TRUE;
      wbs=(struct WBStartup *)argv;
      wba=( wbs->sm_ArgList)+1;
      for ( i= wbs->sm_NumArgs-1; i; i--, wba++) {
	CurrentDir(wba->wa_Lock);
	PlayDT(&ld,wba->wa_Name);
      } // for
    } else { // run from shell
      ULONG opts[OPT_COUNT];
      struct RDArgs *rdargs;
      char *curarg, **argptr;

      memset (opts, 0, sizeof(opts));
      if (rdargs = ReadArgs (TEMPLATE, (LONG *)opts, NULL)) {
	argptr=(char **)opts[OPT_FILE];
	while (curarg=*argptr++) PlayDT(&ld,curarg);

	FreeArgs (rdargs);
      } else {
	PrintFault (IoErr (), MSG_FAILED);
      } // if rdargs
    } // if 0==argc
  } else { 
    PrintFault (IoErr (), MSG_FAILED);
  } // if Open DataTypesBase Library

  if (ld.DataTypesBase) CloseLibrary(ld.DataTypesBase);
} // main

/******** End of file */

